
import ee from "~/event-emitter";
import { toArray } from "~/dom-methods";
import throttle from "lodash/throttle";

const accordionControlSelector = ".c-control-accordion--footer";
const accordionSelector = ".c-footer-accordion";
const accordionOpenClass = "c-footer-accordion__item--open";
const hideArrowsIfNoChildren = true;

let accordionControl;
let selectedItem;

/**
 * @method init
 * @description init method, adds event listener for scroll and resize
 * @return {void}
 */
export default function init() {
  accordionControl = toArray(document.querySelectorAll(accordionControlSelector));

  onMatchMediaOrientation();

  accordionControl.forEach((el) => {
    if (hideArrowsIfNoChildren) {
      const accordionItem = el.parentElement.parentElement;
      const subAccordionList = accordionItem.querySelector(accordionSelector);
      const hasSubAccordion = subAccordionList !== null;
      if (!hasSubAccordion) {
        accordionItem.classList.add("c-footer-accordion__item--empty");
      }
    }
  });

  const accordionContainer = document.querySelector(".c-footer-accordion--top-level");
  accordionContainer.addEventListener("click", onAccordionContainerClicked);

  window.addEventListener("orientationchange", throttle(onMatchMediaOrientation, 50));
  window.addEventListener("resize", throttle(onMatchMediaOrientation, 50));

  ee.on("footer-accordion:clicked", () => {
    resetContainerHeight();
  });
}

/**
 * @method onAccordionContainerClicked
 * @description callback for clicking on the accordion container
 * @param e {MouseEvent} click event
 * @return {void}
 */
export function onAccordionContainerClicked(e) {
  const targetElement = e.target.parentElement;
  const isAccordionButton = targetElement.classList.contains("c-control-accordion--footer");
  isAccordionButton && onAccordionClicked(targetElement, e);
}

/**
 * @method onMatchMediaOrientation
 * @description track to orientation status for blowout menu
 * @return {void}
 */
export function onMatchMediaOrientation(optionalDataObjectArrayOrWhatever) {
  const footerWrapper = document.querySelector(".c-footer-wrapper");
  const footerColumnMenu = document.querySelector(".c-footer-column");
  const footerAccordionMenu = document.querySelector(accordionSelector);

  if (window.matchMedia("(min-width: 940px)").matches) {
    footerWrapper.classList.add("c-footer-wrapper--column");
    footerColumnMenu.classList.remove("u-hidden");
    footerAccordionMenu.classList.add("u-hidden");
  } else {
    footerWrapper.classList.remove("c-footer-wrapper--column");
    footerColumnMenu.classList.add("u-hidden");
    footerAccordionMenu.classList.remove("u-hidden");
  }
}

/**
 * @method closeAllItems
 * @description removes all open classes
 * @return {void}
 */
export function closeAllItems() {
  const accordionItem = toArray(document.querySelectorAll(".c-footer-accordion__item"));

  accordionItem.forEach((el) => {

    // only close accordion items that is not currently in focus
    el !== selectedItem && el.classList.remove(accordionOpenClass);
  });
}

/**
 * @method isRootAccordionItem
 * @description if element item contains an accordion level 2 return true
 * @param accordionItem {HTMLElement}
 * @return {boolean}
 */
export function isRootAccordionItem(accordionItem) {
  return accordionItem.querySelector(".c-footer-accordion--level-2") !== null;
}

/**
 * @method resetContainerHeight
 * @description calculate the height of the ul and set that to the container
 * @return {void}
 */
export function resetContainerHeight() {
  const accordionTopLevel = document.querySelector(".c-footer-accordion--top-level");
  const categoryContainer = document.querySelector(".c-footer-accordion-category");
  const accordionRect = accordionTopLevel.getBoundingClientRect();
  const accordionHeight = accordionRect.height + "px";
  const documentWidth = document.documentElement.clientWidth;

  if (documentWidth >= 980) {
    categoryContainer.style.height = accordionHeight;
  } else {
    categoryContainer.style.height = "auto";
  }
}

/**
 * @method onAccordionClicked
 * @description on accordion clicked
 * @param el {HTMLElement} c-control--accordion element
 * @param e {MouseEvent} click event
 * @return {void}
 */
export function onAccordionClicked(el, e) {
  const accordionItem = el.parentElement.parentElement;
  const subAccordionList = accordionItem.querySelector(accordionSelector);
  const buttonElement = el.querySelector(".c-footer-accordion-button");
  const accordionCategory = el.querySelector(".c-footer-accordion-category");
  const hasSubAccordion = subAccordionList !== null;
  const isOpen = hasSubAccordion && accordionItem.classList.contains(accordionOpenClass);

  // if clicking on a root accordion item, then close all other root level items
  if (isRootAccordionItem(accordionItem)) {
    selectedItem = accordionItem;
    closeAllItems();
  }

  if (hasSubAccordion && !isOpen) {
    accordionItem.classList.add(accordionOpenClass);
    buttonElement.setAttribute("aria-expanded", true);
    accordionCategory.innerHTML = "close sub categories";
  } else {
    accordionItem.classList.remove(accordionOpenClass);
    buttonElement.setAttribute("aria-expanded", false);
    accordionCategory.innerHTML = "open sub categories";
  }

  e.stopPropagation();

  ee.emit("footer-accordion:clicked");
}
